# Biblioteca de Sistemas Nao-Lineares
# Aula de Mehtodos Numehricos 2021
#
# Prof. Breno Imbiriba

# Esta biblioteca requer o pacote 
# numpy

####################################
# Metodo da Bissetriz
def bissetriz(f, a, b, tol):
    """Aplica o método da Bissetriz para encontra o zero da função f(x).

       f  - função de uma variável real: y=f(x).
       a  - ponto 'x' mais a esquerda.
       b  - ponto 'x' mais a direita.
       tol- valor de tolerância.
    """ 

    for j in range(0,50):
        x=(a+b)/2
        if f(a)*f(x)<0:
            b=x
        else:
            a=x

        #print([a,b])
 
        if abs(b-a)<tol:
            return (b+a)/2

    print('Cheguei no final de 50 iterações...');
    return (b+a)/2

####################################


####################################
# Método da Newton
def newton(f,df,x0,tol, imet):
    """Aplica o método de Newton para encontra o zero da função f(x).

       f  - função de uma variável real: y=f(x).
       df - derivada da função f(x): df/dx
       x0 - ponto 'x' inicial.
       tol- valor de tolerância.
       imet- Seleciona qual método de parada usar:
             imet=1 - para quando |f(x)|<e
             imet=2 - para quando |x(n)-x(n-1)|<e
    """ 
    for k in range(0,50):

        x1 = x0-f(x0)/df(x0)

        #print('x=',x0,'dx=',x1-x0)

        if imet==1:
            if abs(f(x1))<tol:
                return x1
        if imet==2:
            if abs(x1-x0)<tol:
                return x1
        x0 = x1
    print('Cheguei no final das 50 iterações...');
    return x0


####################################
# Método da Newton-Secante
def newtonsec(f,df,x0,x1,tol, imet):
    """Aplica o método de Newton para encontra o zero da função f(x).

       f  - função de uma variável real: y=f(x).
       df - derivada da função f(x): df/dx
       x0 - ponto 'x' inicial.
       x1 - ponto 'x' próximo a x0
       tol- valor de tolerância.
       imet- Seleciona qual método de parada usar:
             imet=1 - para quando |f(x)|<e
             imet=2 - para quando |x(n)-x(n-1)|<e
    """ 
    for k in range(0,50):
        
        f0 = f(x0)
        f1 = f(x1)
        df = (f1-f0)/(x1-x0)
        x2 = x1-f(x1)/df

        #print('x=',x0,'dx=',x1-x0)

        if imet==1:
            if abs(f(x2))<tol:
                return x2
        if imet==2:
            if abs(x2-x1)<tol:
                return x2
        x0 = x1
        x1 = x2
    print('Cheguei no final das 50 iterações...');
    return x0



##############
# Newton para sistemas não-lineares
##############


def newtonsys(F,J,x0,n,tol):
    import numpy as np
    import sislinlib as sl
    # Cria vetor e matriz com zeros.
    f = np.zeros(n)
    jac = np.zeros((n,n))

    # Loop de iterações (max 50)
    for k in range(0,50):

        # Calcula valores de f e j
        for i in range(0,n):
            f[i]=F[i](x0)
            for j in range(0,n):
                jac[i,j]=J[i][j](x0)

        # Resolve Jv=-F
        # Ax=B <-> A=J, B=-F
        # Transforma B em triangular (Eliminação Gaussiana)
        A,B=sl.eg(jac,-f)
        # Resolve o sistema trangular.
        v=sl.ts(A,B)

        x1 = x0+v

        print(x1)
        if abs(v).max()<tol:
            return x1

        x0 = x1

    print('Cheguei no final das 50 iterações...');
    return x1

